@@ -4,6 +4,7 @@ import zlib
 
 import numpy as np
 from genomic_regions import GenomicRegion
+from deprecated import deprecated
 
 from ..regions import Genome
 from ..hic import Hic
@@ -50,6 +51,8 @@ def is_juicer(file_name):
         return False
 
 
+@deprecated(reason="Outdated API - use matrix copying instead! (fanc hic "
+                   "--deepcopy on the cmd line or matrix.copy()) in the API")
 def convert_juicer_to_hic(juicer_file, genome_file, resolution, juicer_tools_jar_path=None,
                           norm='NONE', output_file=None, inter_chromosomal=True,
                           chromosomes=None):
@@ -234,6 +237,7 @@ def _read_cstr(f):
             return str(buf)
         else:
             buf = buf + b
+    return buf
 
 
 class LazyJuicerEdge(object):
@@ -274,7 +278,7 @@ class LazyJuicerEdge(object):
 
 
 class JuicerHic(RegionMatrixContainer):
-    def __init__(self, hic_file, resolution=None, mode='r', tmpdir=None, norm='KR'):
+    def __init__(self, hic_file, resolution=None, mode='r', tmpdir=None, norm=None):
         RegionMatrixContainer.__init__(self)
         if '@' in hic_file:
             fields = hic_file.split("@")
@@ -289,6 +293,10 @@ class JuicerHic(RegionMatrixContainer):
                 raise ValueError("Conflicting resolution specifications: "
                                  "{} and {}".format(at_resolution, resolution))
             resolution = str_to_int(at_resolution)
+            
+        if not is_juicer(hic_file):
+            raise ValueError("File {} does not seem to be a .hic "
+                             "file produced with juicer!".format(hic_file))
 
         if tmpdir is None or (isinstance(tmpdir, bool) and not tmpdir):
             self.tmp_file_name = None
@@ -316,15 +324,25 @@ class JuicerHic(RegionMatrixContainer):
             warnings.warn("Mode {} not compatible with JuicerHic. "
                           "File will be opened in read-only mode and "
                           "changes will not be saved to file!")
-
+            
+        with open(self._hic_file, 'rb') as req:
+            req.read(4)  # jump to version location
+            self.version = struct.unpack('<i', req.read(4))[0]
+        
         self._resolution = resolution
-        self._normalisation = norm
+        if norm is None:
+            self._normalisation = 'KR' if self.version < 9 else 'SCALE'
+        else:
+            self._normalisation = norm
         self._unit = 'BP'
         self._mappability = None
-
-        if not is_juicer(hic_file):
-            raise ValueError("File {} does not seem to be a .hic "
-                             "file produced with juicer!".format(hic_file))
+        
+        if self.version > 8:
+            warnings.warn(f"Support for Juicer .hic v{self.version} is still in beta. "
+                          "Please report any issues to https://github.com/vaquerizaslab/fanc/issues/92")
+        elif self.version > 9:
+            raise ValueError(f"Juicer v{self.version} not currently supported. "
+                             "Please report to https://github.com/vaquerizaslab/fanc/issues")
 
     def __enter__(self):
         return self
@@ -344,12 +362,13 @@ class JuicerHic(RegionMatrixContainer):
         """
         if self.tmp_file_name is not None and remove_tmp:
             os.remove(self.tmp_file_name)
-
-    @property
-    def version(self):
-        with open(self._hic_file, 'rb') as req:
-            req.read(4)  # jump to version location
-            return struct.unpack('<i', req.read(4))[0]
+            
+    @staticmethod
+    def _version(req):
+        req.seek(0)
+        
+        req.read(4)  # jump to version location
+        return struct.unpack('<i', req.read(4))[0]
 
     def _master_index(self):
         with open(self._hic_file, 'rb') as req:
@@ -358,14 +377,18 @@ class JuicerHic(RegionMatrixContainer):
 
     @staticmethod
     def _skip_to_attributes(req):
+        version = JuicerHic._version(req)
+        
         req.seek(0)
-
-        # skip magic, version, master
         req.read(16)
 
         # skip genome
         while req.read(1).decode("utf-8") != '\0':
             pass
+        
+        if (version > 8):
+            nvi = struct.unpack('<q',req.read(8))[0]
+            nvisize = struct.unpack('<q',req.read(8))[0]
 
     @staticmethod
     def _skip_to_chromosome_lengths(req):
@@ -383,13 +406,17 @@ class JuicerHic(RegionMatrixContainer):
 
     @staticmethod
     def _skip_to_resolutions(req):
+        version = JuicerHic._version(req)
         JuicerHic._skip_to_chromosome_lengths(req)
 
         n_chromosomes = struct.unpack('<i', req.read(4))[0]
         for _ in range(0, n_chromosomes):
             while req.read(1).decode("utf-8", "backslashreplace") != '\0':
                 pass
-            req.read(4)
+            if (version > 8):
+                req.read(8)
+            else:
+                req.read(4)
 
     @staticmethod
     def _skip_to_chromosome_sites(req):
@@ -424,7 +451,10 @@ class JuicerHic(RegionMatrixContainer):
             n_chromosomes = struct.unpack('<i', req.read(4))[0]
             for _ in range(0, n_chromosomes):
                 name = _read_cstr(req)
-                length = struct.unpack('<i', req.read(4))[0]
+                if self.version > 8:
+                    length = struct.unpack('q',req.read(8))[0]
+                else:
+                    length = struct.unpack('<i', req.read(4))[0]
                 chromosome_lengths[name] = length
 
         return chromosome_lengths
@@ -449,7 +479,10 @@ class JuicerHic(RegionMatrixContainer):
             n_chromosomes = struct.unpack('<i', req.read(4))[0]
             for _ in range(0, n_chromosomes):
                 name = _read_cstr(req)
-                req.read(4)
+                if self.version > 8:
+                    req.read(8)
+                else:
+                    req.read(4)
                 chromosomes.append(name)
 
         return chromosomes
@@ -482,18 +515,24 @@ class JuicerHic(RegionMatrixContainer):
 
     @staticmethod
     def _skip_to_footer(req):
+        version = JuicerHic._version(req)
         req.seek(0)
+        
         req.read(8)  # jump to master index location
         master_index = struct.unpack('<q', req.read(8))[0]
 
         # jump to footer location
         req.seek(master_index)
-        req.read(4)  # skip number of bytes
+        # skip number of bytes
+        if version > 8: 
+            req.read(8)  
+        else:
+            req.read(4)
 
     @staticmethod
     def _skip_to_expected_values(req):
         JuicerHic._skip_to_footer(req)
-
+        
         n_entries = struct.unpack('<i', req.read(4))[0]
         for _ in range(n_entries):
             while req.read(1).decode("utf-8", "backslashreplace") != '\0':
@@ -502,24 +541,29 @@ class JuicerHic(RegionMatrixContainer):
 
     @staticmethod
     def _skip_to_normalised_expected_values(req):
+        version = JuicerHic._version(req)
+        
         JuicerHic._skip_to_expected_values(req)
 
         n_vectors = struct.unpack('<i', req.read(4))[0]
         for _ in range(n_vectors):
-            while req.read(1).decode("utf-8", "backslashreplace") != '\0':
-                pass
-            req.read(4)
-
-            n_values = struct.unpack('<i', req.read(4))[0]
-            for j in range(n_values):
-                req.read(8)
+            key = _read_cstr(req)
+            
+            bin_size = struct.unpack('<i', req.read(4))[0]
+            if version > 8:
+                n_values = struct.unpack('<2i', req.read(8))[0]
+            else:
+                n_values = struct.unpack('<i', req.read(4))[0]
+            skip_size = 4 if version > 8 else 8
+            req.read(skip_size * n_values)
 
             n_scaling_factors = struct.unpack('<i', req.read(4))[0]
-            for _ in range(n_scaling_factors):
-                req.read(12)
-
+            skip_size = 8 if version > 8 else 12
+            req.read(skip_size * n_scaling_factors)
+    
     @staticmethod
     def _skip_to_normalisation_vectors(req):
+        version = JuicerHic._version(req)
         JuicerHic._skip_to_normalised_expected_values(req)
 
         n_vectors = struct.unpack('<i', req.read(4))[0]
@@ -528,15 +572,22 @@ class JuicerHic(RegionMatrixContainer):
                 pass
             while req.read(1).decode("utf-8", "backslashreplace") != '\0':
                 pass
-            req.read(4)
-
-            n_values = struct.unpack('<i', req.read(4))[0]
-            for j in range(n_values):
-                req.read(8)
+            
+            bin_size = struct.unpack('<i', req.read(4))[0]
+            if version > 8:
+                n_values = struct.unpack('<2i', req.read(8))[0]
+            else:
+                n_values = struct.unpack('<i', req.read(4))[0]
+            
+            skip_size = 4 if version > 8 else 8
+            #for _ in range(n_values):
+            req.read(skip_size * n_values)
 
             n_scaling_factors = struct.unpack('<i', req.read(4))[0]
-            for _ in range(n_scaling_factors):
-                req.read(12)
+            skip_size = 8 if version > 8 else 12
+            req.read(skip_size * n_scaling_factors)
+            # for _ in range(n_scaling_factors):
+            #     req.read(skip_size)
 
     def _matrix_positions(self):
         """
@@ -556,7 +607,7 @@ class JuicerHic(RegionMatrixContainer):
             return chromosome_pair_positions
 
     @staticmethod
-    def _expected_value_vectors_from_pos(req, normalisation=None, unit='BP'):
+    def _expected_value_vectors_from_pos(req, normalisation=None, unit='BP', version=8):        
         expected_values = defaultdict(list)
         scaling_factors = defaultdict(dict)
 
@@ -570,23 +621,30 @@ class JuicerHic(RegionMatrixContainer):
                 entry_normalisation = 'NONE'
 
             bin_size = struct.unpack('<i', req.read(4))[0]
-
-            ev = []
-            n_values = struct.unpack('<i', req.read(4))[0]
-
-            for j in range(n_values):
-                v = struct.unpack('<d', req.read(8))[0]
-                ev.append(v)
-
+            if version > 8:
+                n_values = struct.unpack('<2i', req.read(8))[0]
+            else:
+                n_values = struct.unpack('<i', req.read(4))[0]
+            
+            value_size = 4 if version > 8 else 8
+            value_type = '<f' if version > 8 else '<d'
+            ev = np.zeros(n_values)
+            for i in range(n_values):
+                ev[i] = struct.unpack(value_type, req.read(value_size))[0]
+            
             if entry_unit == unit and (normalisation is None or entry_normalisation == normalisation):
                 expected_values[bin_size] = ev
 
             sf = dict()
             n_scaling_factors = struct.unpack('<i', req.read(4))[0]
             for _ in range(n_scaling_factors):
-                chromosome_index = struct.unpack('<i', req.read(4))[0]
-                f = struct.unpack('<d', req.read(8))[0]
-                sf[chromosome_index] = f
+                #req.read(12)
+                chromosome_index = struct.unpack('<i',req.read(4))[0]
+                if version > 8:
+                    v = struct.unpack('<f',req.read(4))[0]
+                else:
+                    v = struct.unpack('<d',req.read(8))[0]
+                sf[chromosome_index] = v
 
             if normalisation is None or entry_normalisation == normalisation:
                 scaling_factors[bin_size] = sf
@@ -617,18 +675,20 @@ class JuicerHic(RegionMatrixContainer):
 
     def expected_value_vectors(self):
         with open(self._hic_file, 'rb') as req:
+            version = JuicerHic._version(req)
             JuicerHic._skip_to_expected_values(req)
 
-            return JuicerHic._expected_value_vectors_from_pos(req)
+            return JuicerHic._expected_value_vectors_from_pos(req, version=version)
 
     def normalised_expected_value_vectors(self, normalisation=None):
         if normalisation is None:
             normalisation = self._normalisation
 
         with open(self._hic_file, 'rb') as req:
+            version = JuicerHic._version(req)
             JuicerHic._skip_to_normalised_expected_values(req)
 
-            return JuicerHic._expected_value_vectors_from_pos(req, normalisation=normalisation)
+            return JuicerHic._expected_value_vectors_from_pos(req, version=version, normalisation=normalisation)
 
     def expected_values(self, selected_chromosome=None, norm=True, *args, **kwargs):
         def _fill_norm_vector(v, n, fill):
@@ -681,6 +741,7 @@ class JuicerHic(RegionMatrixContainer):
 
         existing_normalisations = set()
         with open(self._hic_file, 'rb') as req:
+            version = JuicerHic._version(req)
             JuicerHic._skip_to_normalisation_vectors(req)
 
             n_entries = struct.unpack('<i', req.read(4))[0]
@@ -690,19 +751,25 @@ class JuicerHic(RegionMatrixContainer):
                 entry_unit = _read_cstr(req)
                 entry_resolution = struct.unpack('<i', req.read(4))[0]
                 file_position = struct.unpack('<q', req.read(8))[0]
-                req.read(4)  # skip size in bytes
+                req.read(8 if version > 8 else 4)  # skip size in bytes
                 existing_normalisations.add(entry_normalisation)
-
+                
                 if (entry_chromosome_index == chromosome_index and
                         entry_normalisation == normalisation and
                         entry_resolution == resolution and
                         entry_unit == unit):
                     req.seek(file_position)
-                    vector = []
-                    n_values = struct.unpack('<i', req.read(4))[0]
-                    for _ in range(n_values):
-                        v = struct.unpack('<d', req.read(8))[0]
-                        vector.append(v)
+                    
+                    if version > 8:
+                        n_values = struct.unpack('<2i', req.read(8))[0]
+                    else:
+                        n_values = struct.unpack('<i', req.read(4))[0]
+
+                    value_size = 4 if version > 8 else 8
+                    value_type = '<f' if version > 8 else '<d'
+                    vector = np.zeros(n_values)
+                    for i in range(n_values):
+                        vector[i] = struct.unpack(value_type,req.read(value_size))[0]
 
                     return vector
         
@@ -869,34 +936,46 @@ class JuicerHic(RegionMatrixContainer):
         else:
             x_offset = struct.unpack('<i', block[4:8])[0]
             y_offset = struct.unpack('<i', block[8:12])[0]
-            use_short = not struct.unpack('<b', block[12:13])[0] == 0
-            block_type = struct.unpack('<b', block[13:14])[0]
+            use_short = struct.unpack('<b', block[12:13])[0] == 0
+            use_short_bin_x = True
+            use_short_bin_y = True
+            
+            ix_continue = 13
+            if self.version > 8:
+                use_short_bin_x = struct.unpack('<b', block[13:14])[0] == 0
+                use_short_bin_y = struct.unpack('<b', block[14:15])[0] == 0
+                ix_continue = 15
+                            
+            row_bytes = 2 if use_short_bin_x else 4
+            row_type = '<h' if use_short_bin_x else '<f'
+            col_bytes = 2 if use_short_bin_y else 4
+            col_type = '<h' if use_short_bin_y else '<f'
+            weight_bytes = 2 if use_short else 4
+            weight_type = '<h' if use_short else '<f'
+            
+            block_type = struct.unpack('<b', block[ix_continue:ix_continue+1])[0]
             index = 0
 
             if block_type == 1:
-                row_count = struct.unpack('<h', block[14:16])[0]
-                temp = 16
+                row_count = struct.unpack(row_type, block[ix_continue+1:ix_continue+1+row_bytes])[0]
+                temp = ix_continue + 1 + row_bytes
                 for i in range(row_count):
-                    y_raw = struct.unpack('<h', block[temp:(temp + 2)])[0]
-                    temp += 2
+                    y_raw = struct.unpack(row_type, block[temp:(temp + row_bytes)])[0]
+                    temp += row_bytes
                     y = y_raw + y_offset
-                    col_count = struct.unpack('<h', block[temp:(temp + 2)])[0]
-                    temp += 2
+                    col_count = struct.unpack(col_type, block[temp:(temp + col_bytes)])[0]
+                    temp += col_bytes
                     for j in range(col_count):
-                        x_raw = struct.unpack('<h', block[temp:(temp + 2)])[0]
-                        temp += 2
+                        x_raw = struct.unpack(col_type, block[temp:(temp + col_bytes)])[0]
+                        temp += col_bytes
                         x = x_offset + x_raw
-                        if not use_short:
-                            weight = struct.unpack('<h', block[temp:(temp + 2)])[0]
-                            temp += 2
-                        else:
-                            weight = struct.unpack('<f', block[temp:(temp + 4)])[0]
-                            temp += 4
+                        weight = struct.unpack(weight_type, block[temp:(temp + weight_bytes)])[0]
+                        temp += weight_bytes
                         yield x, y, weight
 
                         index += 1
             elif block_type == 2:
-                temp = 14
+                temp = ix_continue + 1
                 n_points = struct.unpack('<i', block[temp:(temp + 4)])[0]
                 temp += 4
                 w = struct.unpack('<h', block[temp:(temp + 2)])[0]
@@ -906,7 +985,7 @@ class JuicerHic(RegionMatrixContainer):
                     col = i - row * w
                     x = int(x_offset + col)
                     y = int(y_offset + row)
-                    if not use_short:
+                    if use_short:
                         weight = struct.unpack('<h', block[temp:(temp + 2)])[0]
                         temp += 2
                         if weight != -32768:
@@ -918,11 +997,12 @@ class JuicerHic(RegionMatrixContainer):
                         if weight != 0x7fc00000:
                             yield x, y, weight
                             index = index + 1
-
+        
     def _read_matrix(self, region1, region2):
+        version = self.version
         region1 = self._convert_region(region1)
         region2 = self._convert_region(region2)
-
+        
         chromosomes = self._all_chromosomes()
         chromosome1_ix = chromosomes.index(region1.chromosome)
         chromosome2_ix = chromosomes.index(region2.chromosome)
@@ -979,21 +1059,35 @@ class JuicerHic(RegionMatrixContainer):
             row1, row2 = int(region2_bins[0] / block_bin_count), int(region2_bins[1] / block_bin_count)
 
             blocks = set()
-            for r in range(row1, row2 + 1):
-                for c in range(col1, col2 + 1):
-                    block_number = r * block_column_count + c
-                    blocks.add(block_number)
+            if version > 8:
+                translatedLowerPAD = int(row1 + col1 / 2 / block_bin_count)
+                translatedHigherPAD = int(row2 + col2 / 2 / block_bin_count + 1)
+                tranlatedNearerDepth = int(np.log2(1 + abs(row1 - col2) / np.sqrt(2) / block_bin_count))
+                tranlatedFurtherDepth = int(np.log2(1 + abs(row2 - col1) / np.sqrt(2) / block_bin_count))
+                nearerDepth = int(min(tranlatedNearerDepth, tranlatedFurtherDepth))
+                if (row1 > col2 and row2 < col1) or (row2 > col1 and row1 < col2):
+                    nearerDepth = 0
+                furtherDepth = int(max(tranlatedNearerDepth, tranlatedFurtherDepth) + 1)
 
-            if region1.chromosome == region2.chromosome:
-                for r in range(col1, col2 + 1):
-                    for c in range(row1, row2 + 1):
+                for r in range(nearerDepth, furtherDepth + 1):
+                    for c in range(translatedLowerPAD, translatedHigherPAD + 1):
                         block_number = r * block_column_count + c
                         blocks.add(block_number)
+            else:
+                for r in range(row1, row2 + 1):
+                    for c in range(col1, col2 + 1):
+                        block_number = r * block_column_count + c
+                        blocks.add(block_number)
+
+                if region1.chromosome == region2.chromosome:
+                    for r in range(col1, col2 + 1):
+                        for c in range(row1, row2 + 1):
+                            block_number = r * block_column_count + c
+                            blocks.add(block_number)
 
             for block_number in blocks:
                 try:
                     file_position, block_size_in_bytes = block_map[block_number]
-
                     for x, y, weight in self._read_block(req, file_position, block_size_in_bytes):
                         if x < y:
                             if region1_bins[0] <= x < region1_bins[1] - 1 and region2_bins[0] <= y < region2_bins[1] - 1:
@@ -1010,7 +1104,6 @@ class JuicerHic(RegionMatrixContainer):
 
     def _edges_subset(self, key=None, row_regions=None, col_regions=None,
                       lazy=False, *args, **kwargs):
-
         if row_regions[0].chromosome != row_regions[-1].chromosome:
             raise ValueError("Cannot subset rows across multiple chromosomes!")
 
@@ -1038,7 +1131,10 @@ class JuicerHic(RegionMatrixContainer):
                            weight=weight)
         else:
             edge = LazyJuicerEdge(source=0, sink=0, weight=1.0, matrix=self)
+            found = False
             for x, y, weight in self._read_matrix(row_span, col_span):
+                if not found and 101000 < x < 101037:
+                    found = True
                 if x > y:
                     x, y = y, x
                 edge.source, edge.sink, edge.weight = x, y, weight
@@ -1099,4 +1195,5 @@ class JuicerHic(RegionMatrixContainer):
             return np.array([r.valid for r in self.regions(region, lazy=True)])
 
     def bias_vector(self):
-        return np.array([r.bias for r in self.regions(lazy=True)])
+        x = np.array([r.bias for r in self.regions(lazy=True)])
+        return x
